.cam_live_view {
    width: 50%;
    height: 50%;
    position: relative;
    overflow: hidden;
    background-color: #000;
}

.recording-dot {
    height: 10px;
    width: 10px;
    border-radius: 50%;
    display: inline-block;
    position: relative;
    top: -1px;
}

.recording_card,.live_video_card {
    cursor: pointer;
}

.modal-body {
    video {
        width: 100%;
    }
}
